
import java.util.Comparator;
import java.util.List;
import java.util.ArrayList;
import java.util.Random;
import java.util.Collections;

/**
   This version of the Triple class
   implements the Comparable<T> interface.

   A generic sorting algorithm can use
   Triple's compareTo() method to sort
   a list of Triple objects.

   And we can also define a Comparator<Triple> object
   for this Triple class and then use a generic sorting
   method that works with a Comparator<T> object.
*/
public class Triple implements Comparable<Triple>
{
   public int x, y, z;

   public Triple(int x, int y, int z)
   {
      this.x = x;
      this.y = y;
      this.z = z;
   }


   @Override public String toString()
   {
      return "<" + x + ", " + y + ", " + z + ">";
   }


   /**
      If you change the definition of this compareTo() method,
      then you change the way a generic sorting method will sort
      lists of Triple objects.
   */
   @Override public int compareTo(Triple that)
   {
      if (this.x != that.x)
      {
         return this.x - that.x;
      }
      else if (this.y != that.y)
      {
         return this.y - that.y;
      }
      else
      {
         return this.z - that.z;
      }
   }


   /**
      Build a List of random Triple objects.
   */
   public static List<Triple> makeTriples(int n1, int n2)
   {
      final Random generator = new Random();
      final List<Triple> result = new ArrayList<>();
      for (int i = 0; i < n1; ++i)
      {
         final int r1 = generator.nextInt(n2);
         final int r2 = generator.nextInt(n2);
         final int r3 = generator.nextInt(n2);
         final Triple triple = new Triple(r1, r2, r3);
         result.add(triple);
      }
      return result;
   }



   public static void main(String[] args)
   {
      final List<Triple> triples = makeTriples(12, 10); // List of random Triple objects.

      System.out.print( triples );
      System.out.println("\n");

      Collections.sort(triples); // Use Java's built in generic sorting method.

      System.out.print( triples );
      System.out.println("\n");

      Collections.sort(triples, new TripleComparator()); // Use Java's built in generic sorting method.

      System.out.print( triples );
      System.out.println("\n");


      // Profile Java's generic sorting method.
      int size = 1_000_000; // starting size for the lists
      int iterations = 20;  // how many times to increase the list size

      for (int i = 1; i <= iterations; ++i)
      {
         final List<Triple> randTriples = makeTriples(i*size, 1_000_000);

         long startTime = System.currentTimeMillis();
            Collections.sort(randTriples);
         long stopTime = System.currentTimeMillis();
         System.out.println("For n = " + (i*size) + ", time = " + (stopTime - startTime));
      }
   }
}



/**
   A Comparator that compares the total sum of each Triple.
*/
class TripleComparator implements Comparator<Triple>
{
   @Override public int compare(Triple t1, Triple t2)
   {
      return (t1.x + t1.y + t1.z) - (t2.x + t2.y + t2.z);
   }
}
